@@ -15,6 +15,7 @@
 
 #include "rb_cairo.h"
 #include "rb_cairo_private.h"
+#include "rb_cairo_io.h"
 
 #ifdef HAVE_RUBY_ST_H
 #  include <ruby/st.h>
@@ -23,7 +24,6 @@
 #endif
 
 VALUE rb_cCairo_Context;
-VALUE rb_cCairo_Rectangle;
 
 static ID cr_id_surface, cr_id_source;
 static ID cr_id_plus, cr_id_minus, cr_id_multi, cr_id_div;
@@ -33,22 +33,6 @@ static cairo_user_data_key_t cr_object_holder_key;
 
 static VALUE cr_get_current_point (VALUE self);
 
-#if CAIRO_CHECK_VERSION(1, 3, 0)
-static ID at_x, at_y, at_width, at_height;
-
-static VALUE
-cr_rectangle_initialize (VALUE self, VALUE x, VALUE y,
-                         VALUE width, VALUE height)
-{
-  rb_ivar_set (self, at_x, x);
-  rb_ivar_set (self, at_y, y);
-  rb_ivar_set (self, at_width, width);
-  rb_ivar_set (self, at_height, height);
-
-  return Qnil;
-}
-#endif
-
 static inline void
 cr_check_status (cairo_t *context)
 {
@@ -154,10 +138,11 @@ cr_initialize (VALUE self, VALUE target)
   cr = cairo_create (RVAL2CRSURFACE (target));
   cr_check_status (cr);
   rb_ivar_set (self, cr_id_surface, target);
-  cr_set_user_data (cr,
-                    &cr_object_holder_key,
-                    cr_object_holder_new(self),
-                    cr_object_holder_free);
+  if (rb_ivar_defined (target, rb_cairo__io_id_output))
+    cr_set_user_data (cr,
+                      &cr_object_holder_key,
+                      cr_object_holder_new (self),
+                      cr_object_holder_free);
   DATA_PTR (self) = cr;
   if (rb_block_given_p ())
     result = rb_ensure (rb_yield, self, cr_destroy_with_destroy_check, self);
@@ -1533,23 +1518,6 @@ Init_cairo_context (void)
   cr_id_multi = rb_intern ("*");
   cr_id_div = rb_intern ("/");
 
-#if CAIRO_CHECK_VERSION(1, 3, 0)
-  rb_cCairo_Rectangle =
-    rb_define_class_under (rb_mCairo, "Rectangle", rb_cObject);
-  at_x = rb_intern ("@x");
-  at_y = rb_intern ("@y");
-  at_width = rb_intern ("@width");
-  at_height = rb_intern ("@height");
-
-  rb_define_attr (rb_cCairo_Rectangle, "x", CR_TRUE, CR_TRUE);
-  rb_define_attr (rb_cCairo_Rectangle, "y", CR_TRUE, CR_TRUE);
-  rb_define_attr (rb_cCairo_Rectangle, "width", CR_TRUE, CR_TRUE);
-  rb_define_attr (rb_cCairo_Rectangle, "height", CR_TRUE, CR_TRUE);
-
-  rb_define_method (rb_cCairo_Rectangle, "initialize",
-                    cr_rectangle_initialize, 4);
-#endif
-
   rb_cCairo_Context =
     rb_define_class_under (rb_mCairo, "Context", rb_cObject);
 
