@@ -2,10 +2,8 @@
 /*
  * Ruby Cairo Binding
  *
- * $Author: kou $
- * $Date: 2008-11-01 14:23:14 $
- *
- * Copyright 2005-2012 Kouhei Sutou <kou@cozmixng.org>
+ * Copyright 2005-2014 Kouhei Sutou <kou@cozmixng.org>
+ * Copyright 2014 Patrick Hanevold <patrick.hanevold@gmail.com>
  * Copyright 2005 Øyvind Kolås <pippin@freedesktop.org>
  * Copyright 2004-2005 MenTaLguY <mental@rydia.com>
  *
@@ -55,11 +53,9 @@ enum ruby_value_type {
 #  undef T_DATA
 #  include <cairo-quartz.h>
 #  define T_DATA RUBY_T_DATA
-#  ifdef HAVE_RUBY_COCOA
-#    define RB_CAIRO_HAS_QUARTZ_SURFACE
-#    ifdef RB_CAIRO_HAS_QUARTZ_IMAGE_SURFACE_TYPE
-#      define RB_CAIRO_HAS_QUARTZ_IMAGE_SURFACE
-#    endif
+#  define RB_CAIRO_HAS_QUARTZ_SURFACE
+#  ifdef RB_CAIRO_HAS_QUARTZ_IMAGE_SURFACE_TYPE
+#    define RB_CAIRO_HAS_QUARTZ_IMAGE_SURFACE
 #  endif
 #endif
 
@@ -106,6 +102,10 @@ VALUE rb_cCairo_CoglSurface = Qnil;
 static ID cr_id_parse;
 static ID cr_id_size;
 static ID cr_id_set_unit;
+static ID cr_id_x;
+static ID cr_id_y;
+static ID cr_id_width;
+static ID cr_id_height;
 static cairo_user_data_key_t cr_closure_key;
 static cairo_user_data_key_t cr_object_holder_key;
 static cairo_user_data_key_t cr_finished_key;
@@ -571,10 +571,10 @@ cr_surface_map_to_image (int argc, VALUE *argv, VALUE self)
       extents = &extents_value;
       if (rb_cairo__is_kind_of (rb_extents, rb_cCairo_Rectangle))
         {
-          extents->x = NUM2INT (rb_iv_get (rb_extents, "@x"));
-          extents->y = NUM2INT (rb_iv_get (rb_extents, "@y"));
-          extents->width = NUM2INT (rb_iv_get (rb_extents, "@width"));
-          extents->height = NUM2INT (rb_iv_get (rb_extents, "@height"));
+          extents->x = NUM2INT (rb_funcall (rb_extents, cr_id_x, 0));
+          extents->y = NUM2INT (rb_funcall (rb_extents, cr_id_y, 0));
+          extents->width = NUM2INT (rb_funcall (rb_extents, cr_id_width, 0));
+          extents->height = NUM2INT (rb_funcall (rb_extents, cr_id_height, 0));
         }
       else
         {
@@ -1211,7 +1211,10 @@ cr_quartz_surface_initialize (int argc, VALUE *argv, VALUE self)
   cairo_surface_t *surface = NULL;
   cairo_format_t format = CAIRO_FORMAT_ARGB32;
   VALUE arg1, arg2, arg3, rb_width, rb_height;
+#ifdef HAVE_RUBY_COCOA
   static VALUE rb_cOSXCGContextRef = Qnil;
+#endif
+  static VALUE rb_cFFIPointer = Qnil;
 
   rb_scan_args (argc, argv, "21", &arg1, &arg2, &arg3);
 
@@ -1232,20 +1235,45 @@ cr_quartz_surface_initialize (int argc, VALUE *argv, VALUE self)
           format = RVAL2CRFORMAT (arg1);
           break;
         default:
+#ifdef HAVE_RUBY_COCOA
           if (NIL_P (rb_cOSXCGContextRef))
             rb_cOSXCGContextRef =
               rb_const_get (rb_const_get (rb_cObject, rb_intern ("OSX")),
                             rb_intern ("CGContextRef"));
+#endif
 
+          if (NIL_P (rb_cFFIPointer))
+            rb_cFFIPointer =
+              rb_const_get (rb_const_get (rb_cObject, rb_intern ("FFI")),
+                            rb_intern ("Pointer"));
+
+#ifdef HAVE_RUBY_COCOA
           if (RTEST (rb_obj_is_kind_of (arg1, rb_cOSXCGContextRef)))
-            rbobj_to_nsobj (arg1, &objc_object);
+            {
+              rbobj_to_nsobj (arg1, &objc_object);
+            }
           else
-            rb_raise (rb_eArgError,
-                      "invalid argument (expect "
-                      "(width, height), "
-                      "(format, width, height) or "
-                      "(cg_context, width, height)): %s",
-                      rb_cairo__inspect (rb_ary_new3 (3, arg1, arg2, arg3)));
+#endif
+            {
+              if (RTEST (rb_obj_is_kind_of (arg1, rb_cFFIPointer)))
+                {
+                  VALUE rb_objc_pointer;
+                  rb_objc_pointer = rb_funcall (arg1,
+                                                rb_intern ("address"),
+                                                0);
+                  objc_object = NUM2ULONG (rb_objc_pointer);
+                }
+              else
+                {
+                  rb_raise (rb_eArgError,
+                            "invalid argument (expect "
+                            "(width, height), "
+                            "(format, width, height), "
+                            "(cg_context, width, height) or "
+                            "(ffi_pointer, width, height)): %s",
+                            rb_cairo__inspect (rb_ary_new3 (3, arg1, arg2, arg3)));
+                }
+            }
           break;
         }
 
@@ -1893,6 +1921,10 @@ Init_cairo_surface (void)
   cr_id_parse = rb_intern ("parse");
   cr_id_size = rb_intern ("size");
   cr_id_set_unit = rb_intern ("unit=");
+  cr_id_x = rb_intern ("x");
+  cr_id_y = rb_intern ("y");
+  cr_id_width = rb_intern ("width");
+  cr_id_height = rb_intern ("height");
 
   rb_cCairo_Surface =
     rb_define_class_under (rb_mCairo, "Surface", rb_cObject);
